"use strict";
Object.defineProperty(exports, "__esModule", {
    value: true
});
Object.defineProperty(exports, "parseKeyOfTuple", {
    enumerable: true,
    get: ()=>parseKeyOfTuple
});
const _predicateTs = require("../../nodes/predicate.js");
const _errorsTs = require("../../utils/errors.js");
const _genericsTs = require("../../utils/generics.js");
const _definitionTs = require("../definition.js");
const _astTs = require("../string/ast.js");
const parseKeyOfTuple = (def, ctx)=>{
    const resolution = ctx.type.meta.scope.resolveNode((0, _definitionTs.parseDefinition)(def[1], ctx));
    if (!(0, _predicateTs.resolutionExtendsDomain)(resolution, "object")) {
        return (0, _errorsTs.throwParseError)((0, _astTs.writeImplicitNeverMessage)(ctx.path, "keyof"));
    }
    if (resolution.object === true) {
        return (0, _astTs.writeImplicitNeverMessage)(ctx.path, "keyof");
    }
    const branchKeys = [];
    const objectBranches = (0, _genericsTs.listFrom)(resolution.object);
    const initialKeys = getPropsFromBranch(objectBranches[0]);
    for(let i = 1; i < objectBranches.length; i++){
        const keys = getPropsFromBranch(objectBranches[i]);
        if (!keys.length) {
            return (0, _astTs.writeImplicitNeverMessage)(ctx.path, "keyof");
        }
        branchKeys.push(keys);
    }
    const result = [];
    initialKeys.forEach((key)=>{
        const hasKey = branchKeys.every((keySet)=>keySet.includes(key));
        if (hasKey) {
            result.push({
                value: key
            });
        }
    });
    return result.length ? {
        string: result
    } : (0, _astTs.writeImplicitNeverMessage)(ctx.path, "keyof");
};
const getPropsFromBranch = (branch)=>(0, _genericsTs.hasKey)(branch, "props") ? Object.keys(branch.props) : [];
