import type { Morph } from "../parse/tuple/morph.js";
import type { Type } from "../scopes/type.js";
import type { Domain } from "../utils/domains.js";
import type { CollapsibleList, defined, Dict } from "../utils/generics.js";
import type { BranchesComparison } from "./branches.js";
import type { IntersectionResult, KeyIntersectionFn } from "./compose.js";
import { IntersectionState } from "./compose.js";
import type { FlattenContext, ResolvedNode, TraversalEntry } from "./node.js";
import type { FlatRules, LiteralRules, Rules } from "./rules/rules.js";
/** If scope is provided, we also narrow each predicate to match its domain.
 * Otherwise, we use a base predicate for all types, which is easier to
 * manipulate.*/
export type Predicate<domain extends Domain = Domain, $ = Dict> = string extends keyof $ ? true | CollapsibleList<Branch> : true | CollapsibleList<Branch<domain, $>>;
export type Branch<domain extends Domain = Domain, $ = Dict> = Rules<domain, $> | TransformationBranch<domain, $>;
export type TransformationBranch<domain extends Domain = Domain, $ = Dict> = {
    rules: Rules<domain, $>;
    morph?: CollapsibleList<Morph>;
};
export declare const branchIsTransformation: (branch: Branch) => branch is TransformationBranch<keyof {
    bigint: bigint;
    boolean: boolean;
    number: number;
    object: object;
    string: string;
    symbol: symbol;
    undefined: undefined;
    null: null;
}, Dict>;
export type FlatBranch = FlatRules | FlatTransformationBranch;
export type FlatTransformationBranch = [...rules: FlatRules, morph: MorphEntry];
export type MorphEntry = ["morph", CollapsibleList<Morph>];
export type PredicateComparison = IntersectionResult<Predicate> | BranchesComparison;
export declare const comparePredicates: (l: Predicate, r: Predicate, context: IntersectionState) => PredicateComparison;
export declare const predicateIntersection: KeyIntersectionFn<Required<ResolvedNode>>;
export declare const predicateUnion: (domain: Domain, l: Predicate, r: Predicate, type: Type) => true | import("./rules/rules.js").NarrowableRules<Dict> | LiteralRules<keyof {
    bigint: bigint;
    boolean: boolean;
    number: number;
    object: object;
    string: string;
    symbol: symbol;
    undefined: undefined;
    null: null;
}> | TransformationBranch<keyof {
    bigint: bigint;
    boolean: boolean;
    number: number;
    object: object;
    string: string;
    symbol: symbol;
    undefined: undefined;
    null: null;
}, Dict> | readonly Branch<keyof {
    bigint: bigint;
    boolean: boolean;
    number: number;
    object: object;
    string: string;
    symbol: symbol;
    undefined: undefined;
    null: null;
}, Dict>[];
export declare const flattenPredicate: (predicate: Predicate, context: FlattenContext) => TraversalEntry[];
export declare const isLiteralCondition: (predicate: Predicate) => predicate is LiteralRules;
export type DomainSubtypeResolution<domain extends Domain> = {
    readonly [k in domain]: defined<ResolvedNode[domain]>;
};
export declare const resolutionExtendsDomain: <domain extends keyof {
    bigint: bigint;
    boolean: boolean;
    number: number;
    object: object;
    string: string;
    symbol: symbol;
    undefined: undefined;
    null: null;
}>(resolution: ResolvedNode, domain: domain) => resolution is DomainSubtypeResolution<domain>;
