import type { FlatBound } from "../nodes/rules/range.js";
import type { Domain } from "../utils/domains.js";
import { domainOf } from "../utils/domains.js";
import type { constructor, evaluate, evaluateObject, instanceOf } from "../utils/generics.js";
import type { DefaultObjectKind } from "../utils/objectKinds.js";
import { objectKindOf } from "../utils/objectKinds.js";
import { Path } from "../utils/paths.js";
import type { ConstrainedRuleTraversalData, TraversalState } from "./traverse.js";
export declare class ArkTypeError extends TypeError {
    cause: Problems;
    constructor(problems: Problems);
}
export declare class Problem<code extends ProblemCode = any> {
    code: code;
    path: Path;
    protected data: ProblemData<code>;
    protected source: ProblemSource<code>;
    protected writers: ProblemWriters<code>;
    parts?: Problem[];
    constructor(code: code, path: Path, data: ProblemData<code>, source: ProblemSource<code>, writers: ProblemWriters<code>);
    toString(): string;
    get message(): string;
    get reason(): string;
    get mustBe(): string;
}
declare class ProblemArray extends Array<Problem> {
    private state;
    byPath: Record<string, Problem>;
    constructor(state: TraversalState);
    add<code extends ProblemCode>(code: code, data: ProblemData<code>, source: ProblemSource<code>): void;
    get summary(): string;
    toString(): string;
    throw(): never;
}
export declare const Problems: new (state: TraversalState) => readonly Problem[] & {
    [k in Exclude<keyof ProblemArray, keyof unknown[]>]: ProblemArray[k];
};
export type Problems = instanceOf<typeof Problems>;
export declare const describeDomains: (domains: Domain[]) => string;
export declare const describeBranches: (descriptions: string[]) => string;
type ProblemSources = {
    divisor: number;
    class: constructor;
    domain: Domain;
    domainBranches: Domain[];
    objectKind: DefaultObjectKind;
    missing: undefined;
    bound: FlatBound;
    regex: string;
    value: unknown;
    valueBranches: unknown[];
    multi: Problem[];
    branches: Problem[];
};
export type ProblemCode = evaluate<keyof ProblemSources>;
export type ProblemSource<code extends ProblemCode = ProblemCode> = ProblemSources[code];
type ProblemDataByCode = {
    [code in ProblemCode]: code extends keyof ConstrainedRuleTraversalData ? ConstrainedRuleTraversalData[code] : unknown;
};
export type ProblemData<code extends ProblemCode = ProblemCode> = ProblemDataByCode[code];
export type MustBeWriter<code extends ProblemCode> = (source: ProblemSources[code]) => string;
export type ReasonWriter<code extends ProblemCode = ProblemCode> = (mustBe: string, data: DataWrapper<code extends keyof ConstrainedRuleTraversalData ? ConstrainedRuleTraversalData[code] : unknown>) => string;
export type ContextWriter = (reason: string, path: Path) => string;
export declare class DataWrapper<value = unknown> {
    value: value;
    constructor(value: value);
    toString(): string;
    get domain(): domainOf<value>;
    get objectKind(): "non-object" | NonNullable<objectKindOf<value, {
        Array: ArrayConstructor;
        Date: DateConstructor;
        Error: ErrorConstructor;
        Function: FunctionConstructor;
        Map: MapConstructor;
        RegExp: RegExpConstructor;
        Set: SetConstructor;
        Object: ObjectConstructor;
        String: StringConstructor;
        Number: NumberConstructor;
        Boolean: BooleanConstructor;
        WeakMap: WeakMapConstructor;
        WeakSet: WeakSetConstructor;
        Promise: PromiseConstructor;
    }>>;
    get size(): number;
    get units(): "" | "characters" | "items";
    get className(): any;
}
export declare const defaultProblemWriters: DefaultProblemsWriters;
export type ProblemOptions<code extends ProblemCode = ProblemCode> = {
    mustBe?: MustBeWriter<code>;
    writeReason?: ReasonWriter<code>;
    addContext?: ContextWriter;
};
export type ProblemsConfig = evaluateObject<{
    defaults?: ProblemOptions<ProblemCode>;
} & {
    [code in ProblemCode]?: ProblemOptions<code>;
}>;
export type DefaultProblemsWriters = {
    [code in ProblemCode]: ProblemWriters<code>;
};
export type ProblemWriters<code extends ProblemCode> = Required<ProblemOptions<code>>;
export {};
