import type { TraversalEntry, TraversalKey, TraversalNode } from "../nodes/node.js";
import type { Type, TypeConfig } from "../scopes/type.js";
import type { Dict, extend, List } from "../utils/generics.js";
import { Path } from "../utils/paths.js";
import type { SizedData } from "../utils/size.js";
import type { Problem, ProblemCode, ProblemWriters } from "./problems.js";
export declare class TraversalState {
    #private;
    type: Type;
    path: Path;
    problems: readonly Problem<any>[] & {
        byPath: Record<string, Problem<any>>;
        add: <code extends keyof {
            divisor: number;
            class: import("../utils/generics.js").constructor;
            domain: keyof {
                bigint: bigint;
                boolean: boolean;
                number: number;
                object: object;
                string: string;
                symbol: symbol;
                undefined: undefined;
                null: null;
            };
            domainBranches: (keyof {
                bigint: bigint;
                boolean: boolean;
                number: number;
                object: object;
                string: string;
                symbol: symbol;
                undefined: undefined;
                null: null;
            })[];
            objectKind: "Array" | "Date" | "Error" | "Function" | "Map" | "RegExp" | "Set" | "Object" | "String" | "Number" | "Boolean" | "WeakMap" | "WeakSet" | "Promise";
            missing: undefined;
            bound: import("../nodes/rules/range.js").FlatBound;
            regex: string;
            value: unknown;
            valueBranches: unknown[];
            multi: Problem<any>[];
            branches: Problem<any>[];
        }>(code: code, data: import("./problems.js").ProblemData<code>, source: import("./problems.js").ProblemSource<code>) => void;
        summary: string;
        throw: () => never;
    };
    configs: TypeConfig[];
    failFast: boolean;
    constructor(type: Type);
    getConfigForProblemCode<code extends ProblemCode>(code: code): ProblemWriters<code>;
    traverseResolution(data: unknown, name: string): void;
    traverseBranches(data: unknown, branches: TraversalEntry[][]): void;
}
export declare const traverse: (data: unknown, node: TraversalNode, state: TraversalState) => unknown;
export declare const checkEntries: (data: unknown, entries: List<TraversalEntry>, state: TraversalState) => unknown;
export declare const checkObjectKind: TraversalCheck<"objectKind">;
export type TraversalCheck<k extends TraversalKey> = (data: RuleData<k>, rule: Extract<TraversalEntry, [k, unknown]>[1], state: TraversalState) => void;
export type ConstrainedRuleTraversalData = extend<{
    [k in TraversalKey]?: unknown;
}, {
    regex: string;
    divisor: number;
    bound: SizedData;
    requiredProps: Dict;
    optionalProps: Dict;
    objectKind: object;
    class: object;
}>;
export type RuleData<k extends TraversalKey> = k extends keyof ConstrainedRuleTraversalData ? ConstrainedRuleTraversalData[k] : unknown;
